@@ -109,6 +109,10 @@ parseString :: Val -> Parser String
 parseString (getVal -> String x) = return $ T.unpack x
 parseString v = parseFail v "Expected a String"
 
+parseInt :: Val -> Parser Int
+parseInt (getVal -> s@Number{}) = parseJSON s
+parseInt v = parseFail v "Expected an Int"
+
 parseArrayString :: Val -> Parser [String]
 parseArrayString = parseArray >=> mapM parseString
 
@@ -162,10 +166,12 @@ parseConfigYaml v = do
             "group" -> ConfigGroup <$> parseGroup v
             "arguments" -> ConfigSetting . map SettingArgument <$> parseArrayString v
             "fixity" -> ConfigSetting <$> parseFixity v
+            "smell" -> ConfigSetting <$> parseSmell v
             _ | isJust $ getSeverity s -> ConfigGroup . ruleToGroup <$> parseRule o
             _ | Just r <- getRestrictType s -> ConfigSetting . map SettingRestrict <$> (parseArray v >>= mapM (parseRestrict r))
             _ -> parseFail v "Expecting an object with a 'package' or 'group' key, a hint or a restriction"
 
+
 parsePackage :: Val -> Parser Package
 parsePackage v = do
     packageName <- parseField "name" v >>= parseString
@@ -179,6 +185,17 @@ parseFixity v = parseArray v >>= concatMapM (parseHSE parseDeclWithMode >=> f)
         f x@InfixDecl{} = return $ map Infix $ getFixity x
         f _ = parseFail v "Expected fixity declaration"
 
+parseSmell :: Val -> Parser [Setting]
+parseSmell v = do
+  smellName <- parseField "type" v >>= parseString
+  smellType <- require v "Expected SmellType"  $ getSmellType smellName
+  smellLimit <- parseField "limit" v >>= parseInt
+  return [SettingSmell smellType smellLimit]
+    where
+      require :: Val -> String -> Maybe a -> Parser a
+      require _ _ (Just a) = return a
+      require val err Nothing = parseFail val err 
+
 parseGroup :: Val -> Parser Group
 parseGroup v = do
     groupName <- parseField "name" v >>= parseString
