@@ -86,6 +86,9 @@ otherFixities = concat
     ,infix_ 4 ["==="]
     -- esqueleto
     ,infix_ 4 ["==."]
+    -- lattices
+    ,infixr_ 5 ["\\/"] -- \/
+    ,infixr_ 6 ["/\\"] -- /\
     ]
 
 -- Fixites from the `base` package which are currently
@@ -153,7 +156,7 @@ parseModuleEx :: ParseFlags -> FilePath -> Maybe String -> IO (Either ParseError
 parseModuleEx flags file str = timedIO "Parse" file $ do
         str <- case str of
             Just x -> return x
-            Nothing | file == "-" -> getContents
+            Nothing | file == "-" -> getContentsUTF8
                     | otherwise -> readFileUTF8' file
         str <- return $ fromMaybe str $ stripPrefix "\65279" str -- remove the BOM if it exists, see #130
         ppstr <- runCpp (cppFlags flags) file str
