@@ -120,6 +120,13 @@ newtype X = X Int deriving newtype Show
 main = case () of {}
 {-# LANGUAGE EmptyCase #-} \
 main = case () of x -> x --
+{-# LANGUAGE EmptyCase #-} \
+main = case () of x -> x --
+{-# LANGUAGE PolyKinds, KindSignatures #-} -- {-# LANGUAGE PolyKinds #-}
+{-# LANGUAGE PolyKinds, KindSignatures #-} \
+data Set (cxt :: * -> *) a = Set [a] -- @Note Extension KindSignatures is implied by PolyKinds
+{-# LANGUAGE QuasiQuotes, OverloadedStrings #-} \
+main = putStrLn [f|{T.intercalate "blah" []}|]
 </TEST>
 -}
 
@@ -134,32 +141,64 @@ import Data.Ratio
 import Data.Data
 import Refact.Types
 import Data.Semigroup
+import qualified Data.Set as Set
+import qualified Data.Map as Map
 import Prelude
 
 
 extensionsHint :: ModuHint
-extensionsHint _ x = [rawIdea Warning "Unused LANGUAGE pragma" (srcInfoSpan sl)
-          (prettyPrint o) (Just newPragma)
-          (warnings old new) [refact]
-    | not $ used TemplateHaskell x -- if TH is on, can use all other extensions programmatically
-    , o@(LanguagePragma sl exts) <- modulePragmas x
-    , let old = map (parseExtension . prettyPrint) exts
-    , let new = minimalExtensions x old
-    , let newPragma = if null new then "" else prettyPrint $ LanguagePragma sl $ map (toNamed . prettyExtension) new
-    , let refact = ModifyComment (toSS o) newPragma
-    , sort new /= sort old]
+extensionsHint _ x =
+    [ rawIdea Warning "Unused LANGUAGE pragma"
+        (srcInfoSpan sl)
+        (prettyPrint o)
+        (Just newPragma)
+        ( [RequiresExtension $ prettyExtension gone | x <- before \\ after, gone <- Map.findWithDefault [] x disappear] ++
+            [ Note $ "Extension " ++ prettyExtension x ++ " is " ++ reason x
+            | x <- explainedRemovals])
+        [ModifyComment (toSS o) newPragma]
+    | o@(LanguagePragma sl exts) <- modulePragmas x
+    , let before = map (parseExtension . prettyPrint) exts
+    , let after = filter (`Set.member` keep) before
+    , before /= after
+    , let explainedRemovals
+            | null after && not (any (`Map.member` implied) before) = []
+            | otherwise = before \\ after
+    , let newPragma = if null after then "" else prettyPrint $ LanguagePragma sl $ map (toNamed . prettyExtension) after
+    ]
+    where
+        usedTH = used TemplateHaskell x || used QuasiQuotes x
+            -- if TH or QuasiQuotes is on, can use all other extensions programmatically
 
+        -- all the extensions defined to be used
+        extensions = Set.fromList [parseExtension $ fromNamed e | LanguagePragma _ exts <- modulePragmas x, e <- exts]
 
-minimalExtensions :: Module_ -> [Extension] -> [Extension]
-minimalExtensions x es = nubOrd $ concatMap f es
-    where f e = [e | usedExt e x]
+        -- those extensions we detect to be useful
+        useful = if usedTH then extensions  else Set.filter (`usedExt` x) extensions
 
+        -- those extensions which are useful, but implied by other useful extensions
+        implied = Map.fromList
+            [ (e, a)
+            | e <- Set.toList useful
+            , a:_ <- [filter (`Set.member` useful) $ extensionImpliedBy e]]
 
--- RecordWildCards implies DisambiguateRecordFields, but most people probably don't want it
-warnings old new | wildcards `elem` old && wildcards `notElem` new = [RequiresExtension "DisambiguateRecordFields"]
-    where wildcards = EnableExtension RecordWildCards
-warnings _ _ = []
+        -- those we should keep
+        keep =  useful `Set.difference` Map.keysSet implied
 
+        -- (a,b) means a used to imply b, but has gone, so suggest enabling b
+        disappear =
+            Map.fromListWith (++) $
+            nubOrdOn snd -- only keep one instance for each of a
+            [ (e, [a])
+            | e <- Set.toList $ extensions `Set.difference` keep
+            , a <- extensionImplies e
+            , a `Set.notMember` useful
+            , usedTH || usedExt a x
+            ]
+
+        reason x =
+            case Map.lookup x implied of
+            Just a -> "implied by " ++ prettyExtension a
+            Nothing -> "not used"
 
 deriveHaskell = ["Eq","Ord","Enum","Ix","Bounded","Read","Show"]
 deriveGenerics = ["Data","Typeable","Generic","Generic1","Lift"]
@@ -267,6 +306,7 @@ addDerives _ (Just s) xs = case s of
     DerivStock{} -> mempty{derivesStock = xs}
     DerivAnyclass{} -> mempty{derivesAnyclass = xs}
     DerivNewtype{} -> mempty{derivesNewtype = xs}
+    DerivVia{} -> mempty
 addDerives nt _ xs = mempty
     {derivesStock = stock
     ,derivesAnyclass = other
