@@ -42,6 +42,7 @@ import Data.List.Extra
 import Data.Data
 import Data.Char
 import Data.Maybe
+import Refact.Types hiding (RType(Match))
 import qualified Data.Set as Set
 
 
@@ -49,9 +50,11 @@ namingHint :: DeclHint
 namingHint _ modu = naming $ Set.fromList $ concatMap getNames $ moduleDecls modu
 
 naming :: Set.Set String -> Decl_ -> [Idea]
-naming seen x = [suggestN "Use camelCase" x2 (replaceNames res x2) | not $ null res]
+naming seen x = [suggest "Use camelCase" x' x2' [Replace Bind (toSS x) [] (prettyPrint x2)] | not $ null res]
     where res = [(n,y) | n <- nubOrd $ getNames x, Just y <- [suggestName n], not $ y `Set.member` seen]
-          x2 = shorten x
+          x2 = replaceNames res x
+          x' = shorten x
+          x2' = shorten x2
 
 
 shorten :: Decl_ -> Decl_
@@ -71,7 +74,7 @@ getNames x = case x of
     PatBind{} -> name
     TypeDecl{} -> name
     DataDecl _ _ _ _ cons _ -> name ++ [fromNamed x | QualConDecl _ _ _ x <- cons, x <- f x]
-    GDataDecl _ _ _ _ _ cons _ -> name ++ [fromNamed x | GadtDecl _ x _ _ <- cons]
+    GDataDecl _ _ _ _ _ cons _ -> name ++ [fromNamed x | GadtDecl _ x _ _ _ _ <- cons]
     TypeFamDecl{} -> name
     DataFamDecl{} -> name
     ClassDecl{} -> name
@@ -104,6 +107,6 @@ suggestName x
 
 
 replaceNames :: Data a => [(String,String)] -> a -> a
-replaceNames rep = descendBi f
+replaceNames rep = transformBi f
     where f (Ident _ x) = Ident an $ fromMaybe x $ lookup x rep
           f x = x
